@@ -48,6 +48,9 @@ extern "C" {
 #ifdef WITH_AVRESAMPLE
 #pragma comment(lib, "libavresample.a")
 #endif
+#ifdef WITH_SWRESAMPLE
+#pragma comment(lib, "libswresample.a")
+#endif
 
 #ifdef WITH_OPENCORE_AMR_NB
 #ifdef WITH_GCC_LIBAV
