@@ -1,4 +1,4 @@
-//  Copyright (c) 2007-2011 Fredrik Mellbin
+//  Copyright (c) 2007-2015 Fredrik Mellbin
 //
 //  Permission is hereby granted, free of charge, to any person obtaining a copy
 //  of this software and associated documentation files (the "Software"), to deal
@@ -21,9 +21,11 @@
 #include "videosource.h"
 
 #include "indexing.h"
-#include "numthreads.h"
 #include "videoutils.h"
 
+#include <algorithm>
+#include <thread>
+
 namespace {
 void CopyAVPictureFields(AVPicture &Picture, FFMS_Frame &Dst) {
 	for (int i = 0; i < 4; i++) {
@@ -35,7 +37,7 @@ void CopyAVPictureFields(AVPicture &Picture, FFMS_Frame &Dst) {
 // this might look stupid, but we have actually had crashes caused by not checking like this.
 void SanityCheckFrameForData(AVFrame *Frame) {
 	for (int i = 0; i < 4; i++) {
-		if (Frame->data[i] != NULL && Frame->linesize[i] != 0)
+		if (Frame->data[i] != nullptr && Frame->linesize[i] != 0)
 			return;
 	}
 
@@ -88,6 +90,9 @@ FFMS_Frame *FFMS_VideoSource::OutputFrame(AVFrame *Frame) {
 	LocalFrame.TopFieldFirst = Frame->top_field_first;
 	LocalFrame.ColorSpace = OutputColorSpace;
 	LocalFrame.ColorRange = OutputColorRange;
+	LocalFrame.ColorPrimaries = CodecContext->color_primaries;
+	LocalFrame.TransferCharateristics = CodecContext->color_trc;
+	LocalFrame.ChromaLocation = CodecContext->chroma_sample_location;
 
 	LastFrameHeight = CodecContext->height;
 	LastFrameWidth = CodecContext->width;
@@ -98,7 +103,7 @@ FFMS_Frame *FFMS_VideoSource::OutputFrame(AVFrame *Frame) {
 
 FFMS_VideoSource::FFMS_VideoSource(const char *SourceFile, FFMS_Index &Index, int Track, int Threads)
 : Index(Index)
-, CodecContext(NULL)
+, CodecContext(nullptr)
 {
 	if (Track < 0 || Track >= static_cast<int>(Index.size()))
 		throw FFMS_Exception(FFMS_ERROR_INDEX, FFMS_ERROR_INVALID_ARGUMENT,
@@ -119,9 +124,9 @@ FFMS_VideoSource::FFMS_VideoSource(const char *SourceFile, FFMS_Index &Index, in
 	Frames = Index[Track];
 	VideoTrack = Track;
 
-	memset(&VP, 0, sizeof(VP));
-	memset(&LocalFrame, 0, sizeof(LocalFrame));
-	SWS = NULL;
+	VP = {};
+	LocalFrame = {};
+	SWS = nullptr;
 	LastFrameNum = 0;
 	CurrentFrame = 1;
 	DelayCounter = 0;
@@ -144,7 +149,8 @@ FFMS_VideoSource::FFMS_VideoSource(const char *SourceFile, FFMS_Index &Index, in
 	InputColorSpace = AVCOL_SPC_UNSPECIFIED;
 	InputColorRange = AVCOL_RANGE_UNSPECIFIED;
 	if (Threads < 1)
-		DecodingThreads = GetNumberOfLogicalCPUs();
+		// libav current has issues with greater than 16 threads
+		DecodingThreads = (std::min)(std::thread::hardware_concurrency(), 16u);
 	else
 		DecodingThreads = Threads;
 	DecodeFrame = av_frame_alloc();
@@ -233,7 +239,7 @@ void FFMS_VideoSource::DetectInputFormat() {
 void FFMS_VideoSource::ReAdjustOutputFormat() {
 	if (SWS) {
 		sws_freeContext(SWS);
-		SWS = NULL;
+		SWS = nullptr;
 	}
 
 	DetectInputFormat();
@@ -280,7 +286,7 @@ void FFMS_VideoSource::ReAdjustOutputFormat() {
 void FFMS_VideoSource::ResetOutputFormat() {
 	if (SWS) {
 		sws_freeContext(SWS);
-		SWS = NULL;
+		SWS = nullptr;
 	}
 
 	TargetWidth = -1;
@@ -333,7 +339,7 @@ void FFMS_VideoSource::SetVideoProperties() {
 			"Codec returned zero size video");
 
 	// attempt to correct framerate to the proper NTSC fraction, if applicable
-	CorrectNTSCRationalFramerate(&VP.FPSNumerator, &VP.FPSDenominator);
+	CorrectRationalFramerate(&VP.FPSNumerator, &VP.FPSDenominator);
 	// correct the timebase, if necessary
 	CorrectTimebase(&VP, &Frames.TB);
 
